@@ -1,7 +1,7 @@
 /** @file matchspy.cc
  * @brief MatchSpy implementation.
  */
-/* Copyright (C) 2007,2008,2009,2010,2013,2014,2015 Olly Betts
+/* Copyright (C) 2007,2008,2009,2010,2011,2012,2013,2014,2015 Olly Betts
  * Copyright (C) 2007,2009 Lemur Consulting Ltd
  * Copyright (C) 2010 Richard Boulton
  *
@@ -34,9 +34,9 @@
 
 #include "autoptr.h"
 #include "debuglog.h"
-#include "net/length.h"
+#include "noreturn.h"
 #include "omassert.h"
-#include "serialise.h"
+#include "net/length.h"
 #include "stringutils.h"
 #include "str.h"
 #include "termlist.h"
@@ -46,6 +46,7 @@
 
 using namespace std;
 using namespace Xapian;
+using Xapian::Internal::intrusive_ptr;
 
 MatchSpy::~MatchSpy() {}
 
@@ -94,7 +95,7 @@ class ValueCountTermList : public TermList {
   private:
     map<string, Xapian::doccount>::const_iterator it;
     bool started;
-    Xapian::Internal::RefCntPtr<Xapian::ValueCountMatchSpy::Internal> spy;
+    intrusive_ptr<Xapian::ValueCountMatchSpy::Internal> spy;
   public:
 
     explicit ValueCountTermList(ValueCountMatchSpy::Internal * spy_)
@@ -297,7 +298,7 @@ get_most_frequent_items(vector<StringAndFrequency> & result,
 }
 
 void
-ValueCountMatchSpy::operator()(const Document &doc, weight) {
+ValueCountMatchSpy::operator()(const Document &doc, double) {
     Assert(internal.get());
     ++(internal->total);
     string val(doc.get_value(internal->slot));
@@ -384,17 +385,19 @@ ValueCountMatchSpy::merge_results(const string & s) {
 
     map<string, doccount>::size_type items;
     decode_length(&p, end, items);
-    while (p != end) {
-	while (items != 0) {
-	    size_t vallen;
-	    decode_length_and_check(&p, end, vallen);
-	    string val(p, vallen);
-	    p += vallen;
-	    doccount freq;
-	    decode_length(&p, end, freq);
-	    internal->values[val] += freq;
-	    --items;
-	}
+    while (items != 0) {
+	size_t vallen;
+	decode_length_and_check(&p, end, vallen);
+	string val(p, vallen);
+	p += vallen;
+	doccount freq;
+	decode_length(&p, end, freq);
+	internal->values[val] += freq;
+	--items;
+    }
+    if (p != end) {
+	throw NetworkError("Junk at end of serialised ValueCountMatchSpy "
+			   "results");
     }
 }
 
