@@ -86,6 +86,11 @@ sub_docid(Xapian::docid did, size_t n_dbs)
 
 namespace Xapian {
 
+Database::Database(Database&&) = default;
+
+Database&
+Database::operator=(Database&&) = default;
+
 Database::Database()
 {
     LOGCALL_CTOR(API, "Database", NO_ARGS);
@@ -366,12 +371,13 @@ Database::get_value_lower_bound(Xapian::valueno slot) const
 
     if (rare(internal.empty())) RETURN(string());
 
-    vector<intrusive_ptr<Database::Internal> >::const_iterator i;
-    i = internal.begin();
-    string full_lb = (*i)->get_value_lower_bound(slot);
-    while (++i != internal.end()) {
-	string lb = (*i)->get_value_lower_bound(slot);
-	if (lb < full_lb) full_lb = lb;
+    string full_lb;
+    for (auto&& subdb : internal) {
+	string lb = subdb->get_value_lower_bound(slot);
+	if (lb.empty())
+	    continue;
+	if (full_lb.empty() || lb < full_lb)
+	    full_lb = std::move(lb);
     }
     RETURN(full_lb);
 }
@@ -966,7 +972,11 @@ WritableDatabase::replace_document(const std::string & unique_term,
     // If no unique_term in the database, this is just an add_document().
     if (postit == postlist_end(unique_term)) {
 	// Which database will the next never used docid be in?
-	size_t i = sub_db(get_lastdocid() + 1, n_dbs);
+	Xapian::docid did = get_lastdocid() + 1;
+	if (rare(did == 0)) {
+	    throw Xapian::DatabaseError("Run out of docids - you'll have to use copydatabase to eliminate any gaps before you can add more documents");
+	}
+	size_t i = sub_db(did, n_dbs);
 	RETURN(internal[i]->add_document(document));
     }
 
