@@ -767,7 +767,9 @@ QueryPostingSource::postlist(QueryOptimiser * qopt, double factor) const
     // be called on the Database::Internal object.
     const Xapian::Database wrappeddb(
 	    const_cast<Xapian::Database::Internal*>(&(qopt->db)));
-    RETURN(new ExternalPostList(wrappeddb, source.get(), factor, qopt->matcher));
+    RETURN(new ExternalPostList(wrappeddb, source.get(), factor,
+				qopt->matcher,
+				qopt->shard_index));
 }
 
 PostingIterator::Internal *
