@@ -44,91 +44,91 @@ using namespace std;
 // Class representing a posting (a term/doc pair, and
 // all the relevant positional information, is a single posting)
 class InMemoryPosting {
-    public:
-	Xapian::docid did;
-	bool valid;
-	vector<Xapian::termpos> positions; // Sorted vector of positions
-	Xapian::termcount wdf;
+  public:
+    Xapian::docid did;
+    bool valid;
+    vector<Xapian::termpos> positions; // Sorted vector of positions
+    Xapian::termcount wdf;
 
-	// Merge two postings (same term/doc pair, new positional info)
-	void merge(const InMemoryPosting & post) {
-	    Assert(did == post.did);
+    // Merge two postings (same term/doc pair, new positional info)
+    void merge(const InMemoryPosting & post) {
+	Assert(did == post.did);
 
-	    positions.insert(positions.end(),
-			     post.positions.begin(),
-			     post.positions.end());
-	    // FIXME - inefficient - use merge (and list<>)?
-	    sort(positions.begin(), positions.end());
-	}
+	positions.insert(positions.end(),
+			 post.positions.begin(),
+			 post.positions.end());
+	// FIXME - inefficient - use merge (and list<>)?
+	sort(positions.begin(), positions.end());
+    }
 };
 
 class InMemoryTermEntry {
-    public:
-	string tname;
-	vector<Xapian::termpos> positions; // Sorted vector of positions
-	Xapian::termcount wdf;
+  public:
+    string tname;
+    vector<Xapian::termpos> positions; // Sorted vector of positions
+    Xapian::termcount wdf;
 
-	// Merge two postings (same term/doc pair, new positional info)
-	void merge(const InMemoryTermEntry & post) {
-	    Assert(tname == post.tname);
+    // Merge two postings (same term/doc pair, new positional info)
+    void merge(const InMemoryTermEntry & post) {
+	Assert(tname == post.tname);
 
-	    positions.insert(positions.end(),
-			     post.positions.begin(),
-			     post.positions.end());
-	    // FIXME - inefficient - use merge (and list<>)?
-	    sort(positions.begin(), positions.end());
-	}
+	positions.insert(positions.end(),
+			 post.positions.begin(),
+			 post.positions.end());
+	// FIXME - inefficient - use merge (and list<>)?
+	sort(positions.begin(), positions.end());
+    }
 };
 
 // Compare by document ID
 class InMemoryPostingLessThan {
-    public:
-	int operator() (const InMemoryPosting &p1,
-			const InMemoryPosting &p2) const
-	{
-	    return p1.did < p2.did;
-	}
+  public:
+    int operator() (const InMemoryPosting &p1,
+		    const InMemoryPosting &p2) const
+    {
+	return p1.did < p2.did;
+    }
 };
 
 // Compare by termname
 class InMemoryTermEntryLessThan {
-    public:
-	int operator() (const InMemoryTermEntry&p1,
-			const InMemoryTermEntry&p2) const
-	{
-	    return p1.tname < p2.tname;
-	}
+  public:
+    int operator() (const InMemoryTermEntry&p1,
+		    const InMemoryTermEntry&p2) const
+    {
+	return p1.tname < p2.tname;
+    }
 };
 
 // Class representing a term and the documents indexing it
 class InMemoryTerm {
-    public:
-	// Sorted list of documents indexing this term.
-	vector<InMemoryPosting> docs;
+  public:
+    // Sorted list of documents indexing this term.
+    vector<InMemoryPosting> docs;
 
-	Xapian::termcount term_freq;
-	Xapian::termcount collection_freq;
+    Xapian::termcount term_freq;
+    Xapian::termcount collection_freq;
 
-	InMemoryTerm() : term_freq(0), collection_freq(0) {}
+    InMemoryTerm() : term_freq(0), collection_freq(0) {}
 
-	void add_posting(const InMemoryPosting & post);
+    void add_posting(const InMemoryPosting & post);
 };
 
 /// Class representing a document and the terms indexing it.
 class InMemoryDoc {
-    public:
-	bool is_valid;
-	// Sorted list of terms indexing this document.
-	vector<InMemoryTermEntry> terms;
+  public:
+    bool is_valid;
+    // Sorted list of terms indexing this document.
+    vector<InMemoryTermEntry> terms;
 
-	/* Initialise invalid by default, so that resizing the termlist array
-	 * doesn't create valid documents. */
-	InMemoryDoc() : is_valid(false) {}
+    /* Initialise invalid by default, so that resizing the termlist array
+     * doesn't create valid documents. */
+    InMemoryDoc() : is_valid(false) {}
 
-	// Initialise specifying validity.
-	explicit InMemoryDoc(bool is_valid_) : is_valid(is_valid_) {}
+    // Initialise specifying validity.
+    explicit InMemoryDoc(bool is_valid_) : is_valid(is_valid_) {}
 
-	void add_posting(const InMemoryTermEntry & post);
+    void add_posting(const InMemoryTermEntry& post);
 };
 
 class InMemoryDatabase;
@@ -137,102 +137,111 @@ class InMemoryDatabase;
  */
 class InMemoryPostList : public LeafPostList {
     friend class InMemoryDatabase;
-    private:
-	vector<InMemoryPosting>::const_iterator pos;
-	vector<InMemoryPosting>::const_iterator end;
-	Xapian::doccount termfreq;
-	bool started;
 
-	/** List of positions of the current term.
-	 *  This list is populated when read_position_list() is called.
-	 */
-	InMemoryPositionList mypositions;
+  private:
+    vector<InMemoryPosting>::const_iterator pos;
+    vector<InMemoryPosting>::const_iterator end;
+    Xapian::doccount termfreq;
+    bool started;
 
-	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
+    /** List of positions of the current term.
+     *  This list is populated when read_position_list() is called.
+     */
+    InMemoryPositionList mypositions;
 
-	InMemoryPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
-			 const InMemoryTerm & imterm, const std::string & term_);
-    public:
-	Xapian::doccount get_termfreq() const;
+    Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
 
-	Xapian::docid get_docid() const;     // Gets current docid
-	Xapian::termcount get_doclength() const; // Length of current document
-	Xapian::termcount get_unique_terms() const; // number of terms in current document
-	Xapian::termcount get_wdf() const;	   // Within Document Frequency
-	PositionList * read_position_list();
-	PositionList * open_position_list() const;
+    InMemoryPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
+		     const InMemoryTerm & imterm, const std::string & term_);
+  public:
+    Xapian::doccount get_termfreq() const;
 
-	PostList *next(double w_min); // Moves to next docid
+    Xapian::docid get_docid() const;     // Gets current docid
+    Xapian::termcount get_doclength() const; // Length of current document
+    Xapian::termcount get_unique_terms() const; // number of terms in current document
+    Xapian::termcount get_wdf() const;	   // Within Document Frequency
+    PositionList * read_position_list();
+    PositionList * open_position_list() const;
 
-	PostList *skip_to(Xapian::docid did, double w_min); // Moves to next docid >= specified docid
+    PostList *next(double w_min); // Moves to next docid
 
-	// True if we're off the end of the list.
-	bool at_end() const;
+    // Moves to next docid >= specified docid
+    PostList *skip_to(Xapian::docid did, double w_min);
 
-	string get_description() const;
+    // True if we're off the end of the list.
+    bool at_end() const;
+
+    string get_description() const;
 };
 
 /** A PostList over all docs in an inmemory database.
  */
 class InMemoryAllDocsPostList : public LeafPostList {
     friend class InMemoryDatabase;
-    private:
-	Xapian::docid did;
 
-	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
+  private:
+    Xapian::docid did;
 
-	InMemoryAllDocsPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db);
-    public:
-	Xapian::doccount get_termfreq() const;
+    Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
 
-	Xapian::docid get_docid() const;     // Gets current docid
-	Xapian::termcount get_doclength() const; // Length of current document
-	Xapian::termcount get_unique_terms() const; // number of terms in current document
-	Xapian::termcount get_wdf() const;       // Within Document Frequency
-	PositionList * read_position_list();
-	PositionList * open_position_list() const;
+    InMemoryAllDocsPostList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db);
 
-	PostList *next(double w_min);      // Moves to next docid
+  public:
+    Xapian::doccount get_termfreq() const;
 
-	PostList *skip_to(Xapian::docid did, double w_min); // Moves to next docid >= specified docid
+    Xapian::docid get_docid() const;     // Gets current docid
+    Xapian::termcount get_doclength() const; // Length of current document
+    // number of terms in current document
+    Xapian::termcount get_unique_terms() const;
+    Xapian::termcount get_wdf() const;       // Within Document Frequency
+    PositionList * read_position_list();
+    PositionList * open_position_list() const;
 
-	// True if we're off the end of the list
-	bool at_end() const;
+    PostList *next(double w_min);      // Moves to next docid
 
-	string get_description() const;
+    // Moves to next docid >= specified docid
+    PostList *skip_to(Xapian::docid did, double w_min);
+
+    // True if we're off the end of the list
+    bool at_end() const;
+
+    string get_description() const;
 };
 
 // Term List
 class InMemoryTermList : public TermList {
     friend class InMemoryDatabase;
-    private:
-	vector<InMemoryTermEntry>::const_iterator pos;
-	vector<InMemoryTermEntry>::const_iterator end;
-	Xapian::termcount terms;
-	bool started;
 
-	Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
-	Xapian::docid did;
-	Xapian::termcount document_length;
+  private:
+    vector<InMemoryTermEntry>::const_iterator pos;
+    vector<InMemoryTermEntry>::const_iterator end;
+    Xapian::termcount terms;
+    bool started;
 
-	InMemoryTermList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
-			 Xapian::docid did,
-			 const InMemoryDoc & doc,
-			 Xapian::termcount len);
-    public:
-	Xapian::termcount get_approx_size() const;
+    Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db;
+    Xapian::docid did;
+    Xapian::termcount document_length;
 
-	/// Collate weighting information for the current term.
-	void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
+    InMemoryTermList(Xapian::Internal::intrusive_ptr<const InMemoryDatabase> db,
+		     Xapian::docid did,
+		     const InMemoryDoc & doc,
+		     Xapian::termcount len);
 
-	string get_termname() const;
-	Xapian::termcount get_wdf() const; // Number of occurrences of term in current doc
-	Xapian::doccount get_termfreq() const;  // Number of docs indexed by term
-	TermList * next();
-	TermList * skip_to(const std::string & term);
-	bool at_end() const;
-	Xapian::termcount positionlist_count() const;
-	Xapian::PositionIterator positionlist_begin() const;
+  public:
+    Xapian::termcount get_approx_size() const;
+
+    /// Collate weighting information for the current term.
+    void accumulate_stats(Xapian::Internal::ExpandStats & stats) const;
+
+    string get_termname() const;
+    // Number of occurrences of term in current doc
+    Xapian::termcount get_wdf() const;
+    Xapian::doccount get_termfreq() const;  // Number of docs indexed by term
+    TermList * next();
+    TermList * skip_to(const std::string & term);
+    bool at_end() const;
+    Xapian::termcount positionlist_count() const;
+    Xapian::PositionIterator positionlist_begin() const;
 };
 
 class InMemoryDocument;
@@ -274,8 +283,9 @@ class InMemoryDatabase : public Xapian::Database::Internal {
     Xapian::docid make_doc(const string & docdata);
 
     /* The common parts of add_doc and replace_doc */
-    void finish_add_doc(Xapian::docid did, const Xapian::Document &document);
-    void add_values(Xapian::docid did, const map<Xapian::valueno, string> &values_);
+    void finish_add_doc(Xapian::docid did, const Xapian::Document& document);
+    void add_values(Xapian::docid did,
+		    const map<Xapian::valueno, string>& values_);
 
     void make_posting(InMemoryDoc * doc,
 		      const string & tname,
@@ -334,7 +344,8 @@ class InMemoryDatabase : public Xapian::Database::Internal {
 
     LeafPostList * open_post_list(const string & tname) const;
     TermList * open_term_list(Xapian::docid did) const;
-    Xapian::Document::Internal * open_document(Xapian::docid did, bool lazy) const;
+    Xapian::Document::Internal* open_document(Xapian::docid did,
+					      bool lazy) const;
 
     std::string get_metadata(const std::string & key) const;
     TermList * open_metadata_keylist(const std::string &prefix) const;
