@@ -765,3 +765,88 @@ DEFINE_TESTCASE(postingsourceclone1, !backend)
 
     return true;
 }
+
+class OnlyTheFirstPostingSource : public Xapian::PostingSource {
+    Xapian::doccount last_docid;
+
+    Xapian::docid did;
+
+    bool allow_clone;
+
+  public:
+    static Xapian::doccount shard_index;
+
+    explicit
+    OnlyTheFirstPostingSource(bool allow_clone_) : allow_clone(allow_clone_) {}
+
+    PostingSource* clone() const {
+	return allow_clone ? new OnlyTheFirstPostingSource(true) : nullptr;
+    }
+
+    void init(const Xapian::Database& db) {
+	did = 0;
+	if (shard_index == 0) {
+	    last_docid = db.get_lastdocid();
+	} else {
+	    last_docid = 0;
+	}
+	++shard_index;
+    }
+
+    Xapian::doccount get_termfreq_min() const { return 0; }
+
+    Xapian::doccount get_termfreq_est() const { return last_docid / 2; }
+
+    Xapian::doccount get_termfreq_max() const { return last_docid; }
+
+    void next(double wt) {
+	(void)wt;
+	++did;
+	if (did > last_docid) did = 0;
+    }
+
+    void skip_to(Xapian::docid to_did, double wt) {
+	(void)wt;
+	did = to_did;
+	if (did > last_docid) did = 0;
+    }
+
+    bool at_end() const {
+	return did == 0;
+    }
+
+    Xapian::docid get_docid() const { return did; }
+
+    string get_description() const { return "OnlyTheFirstPostingSource"; }
+};
+
+Xapian::doccount OnlyTheFirstPostingSource::shard_index;
+
+DEFINE_TESTCASE(postingsourceshardindex1, multi) {
+    Xapian::Database db = get_database("apitest_simpledata");
+
+    OnlyTheFirstPostingSource::shard_index = 0;
+
+    Xapian::Enquire enquire(db);
+    {
+	auto ps = new OnlyTheFirstPostingSource(true);
+	enquire.set_query(Xapian::Query(ps->release()));
+
+	Xapian::MSet mset = enquire.get_mset(0, 10);
+	mset_expect_order(mset, 1, 3, 5);
+    }
+
+    {
+	/* Regression test for bug fixed in 1.4.12 - we should get an exception
+	 * if we use a PostingSource that doesn't support clone() with a multi
+	 * DB.
+	 */
+	auto ps = new OnlyTheFirstPostingSource(false);
+	enquire.set_query(Xapian::Query(ps->release()));
+
+	TEST_EXCEPTION(Xapian::InvalidOperationError,
+		       auto m = enquire.get_mset(0, 10));
+    }
+
+    return true;
+}
