@@ -46,15 +46,15 @@ mset_range_is_same(const Xapian::MSet &mset1, unsigned int first1,
 		   const Xapian::MSet &mset2, unsigned int first2,
 		   unsigned int count)
 {
-    TEST_AND_EXPLAIN(mset1.size() >= first1 + count - 1,
+    TEST_AND_EXPLAIN(mset1.size() >= first1 + count,
 		     "mset1 is too small: expected at least " <<
-		     (first1 + count - 1) << " items, got " <<
-		     mset1.size() << ".");
+		     (first1 + count) << " items, got " <<
+		     mset1.size());
 
-    TEST_AND_EXPLAIN(mset2.size() >= first2 + count - 1,
+    TEST_AND_EXPLAIN(mset2.size() >= first2 + count,
 		     "mset2 is too small: expected at least " <<
-		     (first2 + count - 1) << " items, got " <<
-		     mset2.size() << ".");
+		     (first2 + count) << " items, got " <<
+		     mset2.size());
 
     Xapian::MSetIterator i = mset1[first1];
     Xapian::MSetIterator j = mset2[first2];
